﻿;******************************************************************************
;	A.I.VOICE in the Tray
;
;	「A.I.VOICE」と「PotatoVoiceHub」と「VoiceroidTalkPlusReceiverHub」を
;	非表示にする（≒タスクトレイに収納）するツール。
;
;	読み上げ用に常駐させる場合にタスクバーが埋まる問題対応。
;******************************************************************************

;------------------------------------------------------------------------------
; Autoexec
;------------------------------------------------------------------------------
	;環境設定
	#NoEnv							;環境変数無視
	#NoTrayIcon 						;トレイアイコン非表示起動
	#SingleInstance Force					;多重起動禁止
	SetWorkingDir		, %A_ScriptDir%			;作業ディレクトリ変更
	DetectHiddenWindows	, On				;非表示ウィンドウを検出
	OnExit	, Sub_EndProcessing				;終了処理指定


	;定数用変数
	;ソフト情報設定
	APP_TILTE	:= "A.I.VOICE in the Tray"		;タイトル
	APP_VER		:= "20230914"				;バージョン
	APP_ICON	:= "Icon\A.I.VOICE in the Tray.ico"	;メインアイコン(バックアップ用)
	APP_BACKEND	:= "A.I.VOICE in the Tray Backend.exe"	;バックエンド実行ファイル

	;Iniファイル関係
	INI_FILENAME	:= "Config.ini"				;ファイル名
	INI_SECTION	:= "AIVoice"				;セクション名

	;エディタ名（メニュー項目名用）
	EDITOR_NAME	:= "A.I.VOICE Editor"

	;UIイニシャライズ
	TreyIcon_Init()						;トレイアイコンメニュー初期化
	SetTreyIcon()						;アイコン設定
	Gui_Init()						;GUIウィンドウ初期化

	;設定読込
	editor_path :=  SetEditorPath()				;Editorのパスを読込み

	;バックエンド通信用メッセージハンドラ
	OnMessage( 0x8000, "GetBackend_wHnd")

	;ウィンドウ状態取得
	Gosub , Sub_FindWindowAll

	;タイマセット
	SetTimer, Sub_FindWindowAll, 2000

	;トレイアイコン表示
	Menu, Tray, Icon

	Return


;------------------------------------------------------------------------------
; 終了処理
;------------------------------------------------------------------------------
Sub_EndProcessing:
	Gosub	, Sub_ShowWindowAll	;ウィンドウを表示状態に復帰
	ExitApp				;本終了
	Return


;******************************************************************************
; バックエンド通信用メッセージハンドラ
;******************************************************************************
;------------------------------------------------------------------------------
; バックエンド起動時に送られて来るバックエンドのウィンドウハンドルを受信する。
;------------------------------------------------------------------------------
;	msg		Ox8000
;	wParam		使用しない
;	lParam		バックエンドのウィンドウハンドル。
;	返り値		0固定
;------------------------------------------------------------------------------
GetBackend_wHnd(wParam,lParam,msg,hwnd){
	global hwnd_backend
	SetFormat, INTEGER, H

	hwnd_backend	:= lParam
	StringLower, hwnd_backend, hwnd_backend

	GuiControl	, Text	, CtrlEdit_BackEnd_hWnd	, %hwnd_backend%

	Return 0
}



;******************************************************************************
; GUIウィンドウ周り
;******************************************************************************
;------------------------------------------------------------------------------
; GUIウィンドウ初期化
;------------------------------------------------------------------------------
Gui_Init(){
	;変数はグローバル扱い(コントロール識別用)
	global

	;配置パラメータ(個別でローカル変数とする)
	local GUI_CONTROL_PITCH_X		:= 5
	local GUI_CONTROL_PITCH_Y		:= 2

	local GUI_CONTROL_WIDTH_LABEL		:= 180
	local GUI_CONTROL_WIDTH_HWND		:= 70
	local GUI_CONTROL_WIDTH_STATUS		:= 40
	local GUI_CONTROL_WIDTH_STYLE		:= 80
	local GUI_CONTROL_WIDTH_BUTTON		:= 50
	local GUI_CONTROL_HIGHT			:= 20


	;ラベル
	Gui, Add, Text	, xm ym w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, Application

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT%
			, hWnd

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, Hidden

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, MinMax

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT%
			, Style

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT%
			, ExStyle

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, Tray


	;Editor
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, %EDITOR_NAME%

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Find_Editor gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Hide_Editor gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Show_Editor gSub_ShowWindowButton
			, Show


	;PotatoVoiceHub
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, PotatoVoiceHub

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Find_PotatoVoiceHub gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Hide_PotatoVoiceHub gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Show_PotatoVoiceHub gSub_ShowWindowButton
			, Show


	;Voiceroid Talk Plus Receiver Hub
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, Voiceroid Talk Plus Receiver Hub

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Find_VTPRH gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Hide_VTPRH gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% vCtrlButton_Show_VTPRH gSub_ShowWindowButton
			, Show


	;App
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, App

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_App_hWnd
			, 0


	;BackEnd
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, BackEnd

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_BackEnd_hWnd
			, 0


	;ツールウィンドウ
	Gui,+ToolWindow

	;自身のhWndを取得
	Gui, +LastFound
	WinGet, app_hwnd, ID
	GuiControl	, Text	, CtrlEdit_App_hWnd	, %app_hwnd%

	;ウィンドウ仮表示
	Gui, Show, Hide, ウィンドウ認識状態 - %APP_TILTE%

	Return
}


;------------------------------------------------------------------------------
; GUIウィンドウからの操作
;------------------------------------------------------------------------------
;グローバル変数使いたくないので関数化
FxStringReplace(inputvar, searchtext, replacetext="", replaceall=""){
	StringReplace	, outputvar, inputvar, %searchtext%, %replacetext%, %replaceall%
	Return outputvar
}

;ウィンドウ検索ボタン
Sub_FindWindowButton:
	FindWindow(FxStringReplace( A_GuiControl, "CtrlButton_Find_"))
	Return


;ウィンドウ格納ボタン
Sub_HideWindowButton:
	HideWindow(FxStringReplace( A_GuiControl, "CtrlButton_Hide_"))
	Return


;ウィンドウ復帰ボタン
Sub_ShowWindowButton:
	ShowWindow(FxStringReplace( A_GuiControl, "CtrlButton_Show_"))
	Return



;******************************************************************************
; トレイアイコンメニュー関連
;******************************************************************************
;------------------------------------------------------------------------------
; トレイアイコンメニュー 初期化
;------------------------------------------------------------------------------
TreyIcon_Init(){
	;グローバル変数アクセス権
	global APP_TILTE, EDITOR_NAME

	;タイトル設定
	Menu, Tray, Tip, %APP_TILTE%

	;メニュー項目
	Menu, Tray, NoStandard		;標準項目抹消

	Menu, Tray, Add 	, %EDITOR_NAME%を格納			, Sub_EditorSwitch
	Menu, Tray, Add 	, PotatoVoiceHubを格納			, Sub_PotatoVoiceHubSwitch
	Menu, Tray, Add 	, VoiceroidTalkPlusReceiverHubを格納	, Sub_VTPRHSwitch
	Menu, Tray, Add

	Menu, Tray, Add 	, すべて表示				, Sub_ShowWindowAll
	Menu, Tray, Add 	, すべて格納				, Sub_HideWindowAll
	Menu, Tray, Add

	Menu, Tray, Add 	, %EDITOR_NAME%を起動			, Sub_RunEditor
	Menu, Tray, Add

	Menu, Tray, Add 	, %EDITOR_NAME%のパスを変更		, Sub_ChangeEditorPath
	Menu, Tray, Add 	, トレイアイコンの変更			, Sub_ChangeTreyIcon
	;Menu, Tray, Add 	, 全ウィンドウ再検索			, Sub_FindWindowAll
	Menu, Tray, Add 	, ウィンドウ認識状態			, Sub_Gui_Show
	Menu, Tray, Add 	, ウィンドウ認識状態(バックエンド側)	, Sub_Gui_ShowB
	Menu, Tray, Add 	, ツールを終了				, Sub_End

	Return
}


;------------------------------------------------------------------------------
; トレイアイコンメニュー デフォルト切替
;------------------------------------------------------------------------------
TreyIcon_SetDefault(){
	;グローバル変数アクセス権
	global EDITOR_NAME, editor_path

	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. "Editor"	. "_hWnd"	;hWnd

	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%


	;Editorを認識しているか？
	If ( hwnd )
		Menu, Tray, Default	, %EDITOR_NAME%を格納
	Else{
		If ( FileExist( editor_path ))
			Menu, Tray, Default	, %EDITOR_NAME%を起動
		Else
			Menu, Tray, NoDefault
	}

	Return
}


;------------------------------------------------------------------------------
; トレイアイコンメニューからの操作
;------------------------------------------------------------------------------
;Editorの格納状態切替
Sub_EditorSwitch:
	SwitchWindow("Editor")
	Return

;PotatoVoiceHubの格納状態切替
Sub_PotatoVoiceHubSwitch:
	SwitchWindow("PotatoVoiceHub")
	Return

;VoiceroidTalkPlusReceiverHubの格納状態切替
Sub_VTPRHSwitch:
	SwitchWindow("VTPRH")
	Return

;------------------------------------------------------------------------------
;全ウィンドウ表示
Sub_ShowWindowAll:
	ShowWindow("PotatoVoiceHub")	;PotatoVoiceHub
	ShowWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	ShowWindow("Editor")		;Editor
	Return

;全ウィンドウを隠す
Sub_HideWindowAll:
	HideWindow("Editor")		;Editor
	HideWindow("PotatoVoiceHub")	;PotatoVoiceHub
	HideWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	Return

;------------------------------------------------------------------------------
;Editorの起動
Sub_RunEditor:
	If ( FileExist( editor_path ))
		Run	, %editor_path%
			,
			, UseErrorLevel
	Return

;------------------------------------------------------------------------------
;Editorのパスを変更
Sub_ChangeEditorPath:
	ChangeEditorPath()
	editor_path :=  SetEditorPath()	;Editorのパスを再読込み
	Return

;トレイアイコンの変更
Sub_ChangeTreyIcon:
	ChangeTreyIcon()
	Return

;全ウィンドウ再検索
Sub_FindWindowAll:
	FindWindow("Editor")		;Editor
	FindWindow("PotatoVoiceHub")	;PotatoVoiceHub
	FindWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	TreyIcon_SetDefault()		;タスクトレイアイコンデフォルト切替

	;バックエンド実行中チェック
	If ( !WinExist("ahk_id " . hwnd_backend))
	{
		;再起動させる
		If ( FileExist( APP_BACKEND ))
			Run	, %APP_BACKEND% %app_hwnd%
				,
				, UseErrorLevel

		If ( ErrorLevel == "ERROR" )
			SetTimer, Sub_FindWindowAll, 5000	;キャンセル時のユーザー操作猶予
		Else
			SetTimer, Sub_FindWindowAll, 2000	;通常監視
	}
	Return

;ウィンドウ認識状態表示
Sub_Gui_Show:
	Gui, Show
	Return

;バックエンド側ウィンドウ認識状態表示
Sub_Gui_ShowB:
	SendMessage	, 0x8003			;WM_APP
			, 0				;wParam
			, 0				;lParam
			,				;Control
			, ahk_id %hwnd_backend%
	Return

;ツールを終了
Sub_End:
	ExitApp		;終了処理へ移行
	Return



;******************************************************************************
; トレイアイコン関連
;******************************************************************************
;------------------------------------------------------------------------------
; トレイアイコン設定
;------------------------------------------------------------------------------
SetTreyIcon(){
	;グローバル変数アクセス権
	global INI_FILENAME, INI_SECTION, APP_ICON

	;設定ファイル読込
	IniRead	, tray_icon
		, %INI_FILENAME%
		, %INI_SECTION%
		, Icon

	;存在チェック
	If ( FileExist( tray_icon ) )
		Menu, Tray, Icon, %tray_icon%
	Else
		Menu, Tray, Icon, %APP_ICON%

	Return
}


;------------------------------------------------------------------------------
; トレイアイコン変更
;------------------------------------------------------------------------------
ChangeTreyIcon(){
	;グローバル変数アクセス権
	global INI_FILENAME, INI_SECTION, APP_ICON, hwnd_backend

	;ファイル選択
	FileSelectFile	, tray_icon_new
			, 3
			,
			, アイコンを指定
			, アイコンファイル(*.ico; *.exe)

	;キャンセルチェック
	If ( tray_icon_new == "" )
		Return

	;設定ファイル書込
	IniWrite	, %tray_icon_new%
			, %INI_FILENAME%
			, %INI_SECTION%
			, Icon

	;アイコン再設定
	SetTreyIcon()

	;バックエンドへ設定更新を通達
	SendMessage	, 0x8002			;WM_APP
			, 0				;wParam
			, 0				;lParam
			,				;Control
			, ahk_id %hwnd_backend%

	Return
}



;******************************************************************************
; Editor関連
;******************************************************************************
;------------------------------------------------------------------------------
; INIファイルからEditorのパスを読込み
; 返り値：読み込んだパス
;------------------------------------------------------------------------------
SetEditorPath(){
	;グローバル変数アクセス権
	global INI_FILENAME, INI_SECTION, EDITOR_NAME

	;設定ファイル読込
	IniRead	, editor_path
		, %INI_FILENAME%
		, %INI_SECTION%
		, EditorPath

	;存在チェックとメニュー管理
	If ( FileExist( editor_path ) )
		Menu, Tray, Enable, %EDITOR_NAME%を起動
	Else
		Menu, Tray, Disable, %EDITOR_NAME%を起動

	Return editor_path
}


;------------------------------------------------------------------------------
; Editorのパスを変更
;------------------------------------------------------------------------------
ChangeEditorPath(){
	;グローバル変数アクセス権
	global INI_FILENAME, INI_SECTION, EDITOR_NAME

	;ファイル選択
	FileSelectFile	, editor_path_new
			, 3
			,
			, %EDITOR_NAME%を指定
			, %EDITOR_NAME%(*.exe)

	;キャンセルチェック
	If ( editor_path_new == "" )
		Return

	;設定ファイル書込
	IniWrite	, %editor_path_new%
			, %INI_FILENAME%
			, %INI_SECTION%
			, EditorPath

	Return
}



;******************************************************************************
; その他関数
;******************************************************************************
;------------------------------------------------------------------------------
; 共通処理部
; 個別・一括操作ルーチンから呼ばれるコア部分
;------------------------------------------------------------------------------
;ウィンドウ検索
FindWindow(target_name){
	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	If ( target_name == "Editor" )		;Editor
	{
		Process, Exist, AIVoiceEditor.exe
		hwnd := WinExist( "A.I.VOICE Editor" . " ahk_pid " . ErrorLevel )
	}

	If ( target_name == "PotatoVoiceHub" )	;PotatoVoiceHub
	{
		Process, Exist, PotatoVoiceHub.exe
		hwnd := WinExist( "PotatoVoiceHub" . " ahk_pid " . ErrorLevel )
	}

	If ( target_name == "VTPRH" )		;Voiceroid Talk Plus Receiver Hub
	{
		Process, Exist, VoiceroidTalkPlusReceiverHub.exe
		hwnd := WinExist( "Voiceroid Talk Plus Receiver" . " ahk_pid " . ErrorLevel )
	}


	;書き込む
	GuiControl	, Text	, %ctrl_hwnd%	, %hwnd%


	;そのままウィンドウステータス取得
	GetWindowStatus(target_name)

	;状態補正
	FixWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウステータス取得
;------------------------------------------------------------------------------
GetWindowStatus(target_name){
	;グローバル変数アクセス権
	global EDITOR_NAME

	;変数初期化
	hwnd		:= 0
	hidden		:= 0
	minmax		:= 0
	style		:= 0
	exstyle		:= 0
	tray		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd
	ctrl_hidden	:= "CtrlEdit_"	. target_name	. "_Hidden"	;非表示状態
	ctrl_minmax 	:= "CtrlEdit_"	. target_name	. "_MinMax"	;最大化・最小化状態
	ctrl_style 	:= "CtrlEdit_"	. target_name	. "_Style"	;ウィンドウスタイル
	ctrl_exstyle 	:= "CtrlEdit_"	. target_name	. "_ExStyle"	;ウィンドウ拡張スタイル
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウステータス取得
	If ( hwnd )
	{
		WinGet, minmax	, MinMax	, ahk_id %hwnd%
		WinGet, style	, Style		, ahk_id %hwnd%
		WinGet, exstyle	, ExStyle	, ahk_id %hwnd%

		;非表示状態チェック
		DetectHiddenWindows, Off				;非表示ウィンドウ検出OFF

		If !WinExist( "ahk_id " . hwnd )
			hidden := 1

		DetectHiddenWindows, On					;非表示ウィンドウ検出ON
	}


	;トレイ化状態判別
	;(トレイ化とみなす条件はアプリケーションによって異なる為判別ロジックも異なる。)
	;Editor
	If ( target_name == "Editor" )
	{
		;格納状態判別条件＝最小化＆ツールウィンドウ状態

		;ツールウィンドウか判別
		Transform	, exstyle_toold
				, BitAnd
				, exstyle
				, 0x00000080

		;判定
		If ( minmax == -1 )
		&& ( exstyle_toold )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, %EDITOR_NAME%を格納
		Else
			Menu, Tray, UnCheck, %EDITOR_NAME%を格納
	}

	;PotatoVoiceHub
	If ( target_name == "PotatoVoiceHub" )
	{
		;格納状態判別条件＝最小化＆非表示
		If ( minmax == "-1" )
		&& ( hidden )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, PotatoVoiceHubを格納
		Else
			Menu, Tray, UnCheck, PotatoVoiceHubを格納

	}

	;Voiceroid Talk Plus Receiver Hub
	If ( target_name == "VTPRH" )
	{
		;格納状態判別条件＝非表示
		If ( hidden )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, VoiceroidTalkPlusReceiverHubを格納
		Else
			Menu, Tray, UnCheck, VoiceroidTalkPlusReceiverHubを格納
	}


	;結果を書き込む
	GuiControl	, Text	, %ctrl_hidden%		, %hidden%
	GuiControl	, Text	, %ctrl_minmax%		, %minmax%
	GuiControl	, Text	, %ctrl_style%		, %style%
	GuiControl	, Text	, %ctrl_exstyle%	, %exstyle%
	GuiControl	, Text	, %ctrl_tray%		, %tray%

	Return
}


;------------------------------------------------------------------------------
; 状態補正
; 格納状態で新しく起動したときなどのウィンドウ状態の自動修正や
; 最小化時に自動でトレイアイコン化するなど。
;------------------------------------------------------------------------------
FixWindowStatus(target_name){
	;変数初期化
	hwnd		:= 0
	hidden		:= 0
	minmax		:= 0
	style		:= 0
	exstyle		:= 0
	tray		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd
	ctrl_hidden	:= "CtrlEdit_"	. target_name	. "_Hidden"	;非表示状態
	ctrl_minmax 	:= "CtrlEdit_"	. target_name	. "_MinMax"	;最大化・最小化状態
	ctrl_style 	:= "CtrlEdit_"	. target_name	. "_Style"	;ウィンドウスタイル
	ctrl_exstyle 	:= "CtrlEdit_"	. target_name	. "_ExStyle"	;ウィンドウ拡張スタイル
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウステータス取得
	If ( hwnd )
	{
		GuiControlGet	, hidden	,	, %ctrl_hidden%
		GuiControlGet	, minmax	,	, %ctrl_minmax%
		GuiControlGet	, style		,	, %ctrl_style%
		GuiControlGet	, exstyle	,	, %ctrl_exstyle%
		GuiControlGet	, tray		,	, %ctrl_tray%


		;Editorの復帰漏れ(≒ミス)対策
		If ( target_name == "Editor" )
		{
			;ツールウィンドウか判別
			Transform	, exstyle_toold
					, BitAnd
					, exstyle
					, 0x00000080

			;判定
			If ( minmax != -1 )
			&& ( exstyle_toold )
				ShowWindow(target_name)
		}


		;補助アプリは最小化で非格納なら格納させる。
		If ( target_name == "PotatoVoiceHub" )
		|| ( target_name == "VTPRH" )
		{
			If ( minmax == -1 )
			&& ( tray == 0 )
				HideWindow(target_name)
		}
	}
}


;------------------------------------------------------------------------------
; ウィンドウ格納
;------------------------------------------------------------------------------
HideWindow(target_name){
	;グローバル変数アクセス権
	global hwnd_backend

	;変数初期化
	hwnd		:= 0
	target_number	:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウを隠す
	If ( hwnd )
	&& ( WinExist( "ahk_id " . hwnd_backend ))
	{
		;アプリ個別対応
		If ( target_name == "Editor" )
			target_number	:= 0

		If ( target_name == "PotatoVoiceHub" )
			target_number	:= 1

		If ( target_name == "VTPRH" )
			target_number	:= 2

		SendMessage	, 0x8001			;WM_APP
				, %target_number%		;wParam
				, 0				;lParam
				,				;Control
				, ahk_id %hwnd_backend%
	}

	;ステータス最新化
	GetWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウ格納から復帰
;------------------------------------------------------------------------------
ShowWindow(target_name){
	;グローバル変数アクセス権
	global hwnd_backend

	;変数初期化
	hwnd		:= 0
	target_number	:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウを表示
	If ( hwnd )
	&& ( WinExist( "ahk_id " . hwnd_backend ))
	{
		;アプリ個別対応
		If ( target_name == "Editor" )
			target_number	:= 0

		If ( target_name == "PotatoVoiceHub" )
			target_number	:= 1

		If ( target_name == "VTPRH" )
			target_number	:= 2

		SendMessage	, 0x8001			;WM_APP
				, %target_number%		;wParam
				, 1				;lParam
				,				;Control
				, ahk_id %hwnd_backend%
	}

	;ステータス最新化
	GetWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウ格納状態切替
;------------------------------------------------------------------------------
SwitchWindow(target_name){
	;変数初期化
	tray		:= 0

	;コントロール名
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;ステータス最新化
	GetWindowStatus(target_name)


	;格納状態取得
	GuiControlGet	, tray	,	, %ctrl_tray%

	;格納されているか？
	If ( tray )
		ShowWindow(target_name)		;復帰
	Else
		HideWindow(target_name)		;格納

	Return
}
